window.Gleam.initSearch([{"doc":"snag","title":"snag","content":"# Snag\n\nA Snag is a boilerplate-free ad-hoc error type.\n\nUse `Result(value, Snag)` (or the `snag.Result(value)` alias) in functions\nthat may fail.\n\nA low level message like &quot;Unexpected status 401&quot; or &quot;No such file or\ndirectory&quot; can be confusing or difficult to debug, so use the `snag.context`\nfunction to add extra contextual information.\n\n```gleam\nimport gleam/io\nimport gleam/result\nimport my_app.{User}\nimport snag.{Result}\n\npub fn log_in(user_id: Int) -&gt; Result(User) {\n  use api_key &lt;- result.try(\n    my_app.read_file(&quot;api_key.txt&quot;)\n    |&gt; snag.context(&quot;Could not load API key&quot;)\n  )\n\n  use session_token &lt;- result.try(\n    user_id\n    |&gt; my_app.create_session(api_key)\n    |&gt; snag.context(&quot;Session creation failed&quot;)\n  )\n\n  Ok(session_token)\n}\n\npub fn main() {\n  case log_in(42) {\n    Ok(session) -&gt; io.println(&quot;Logged in!&quot;)\n    Error(snag) -&gt; {\n      io.print(snag.pretty_print(snag))\n      my_app.exit(1)\n    }\n  }\n}\n```\n\nIn this code when an error occurs within the `create_session` function an\nerror message like this is printed using the added contextual information:\n\n```text\nerror: Session creation failed\n\ncause:\n  0: Unable to exchange token with authentication service\n  1: Service authentication failed\n  2: Unexpected HTTP status 401\n```\n\n## When should I use Snag?\n\nSnag is useful in code where it must either pass or fail, and when it fails we\nwant good debugging information to print to the user. i.e. Command line\ntools, data processing pipelines, etc. Here Snag provides a convenient way to\ncreate errors with a reasonable amount of debugging information, without the\nboilerplate of a custom error type.\n\nIt is not suited to code where the application needs to make a decision about\nwhat to do in the event of an error, such as whether to give up or to try\nagain. i.e. Libraries, web application backends, API clients, etc. In these\nsituations it is recommended to create a custom type for your errors as it\ncan be pattern matched on and have any additional detail added as fields.\n\n## Installation\n\nAdd `snag` to your Gleam project\n\n```\ngleam add snag\n```\n\n## Prior art\n\nThis library is inspired by the following projects:\n\n- Rust&#39;s [`anyhow`](https://github.com/dtolnay/anyhow) and\n  [`std::error::Error`](https://doc.rust-lang.org/std/error/trait.Error.html)\n- Go&#39;s [`error`](https://golang.org/pkg/errors/).\n","url":"index.html"},{"doc":"snag","title":"Result","content":"pub type Result(t) =\n  gleam.Result(t, Snag)\n A concise alias for a `Result` that uses a `Snag` as the error value.\n\nSynonyms:\nsnag.Result\nsnag Result","url":"snag.html#Result"},{"doc":"snag","title":"Snag","content":"pub type Snag {\n  Snag(issue: String, cause: List(String))\n}\n A Snag is a boilerplate-free error type that can be used to track why an\n error happened, though does not store as much detail on specific errors as a\n custom error type would.\n\n It is useful in code where it must either pass or fail, and when it fails we\n want good debugging information to print to the user. i.e. Command line\n tools, data processing pipelines, etc.\n\n If it not suited to code where the application needs to make a decision about\n what to do in the event of an error, such as whether to give up or to try\n again. i.e. Libraries, web application backends, API clients, etc.\n In these situations it is recommended to create a custom type for your errors\n as it can be pattern matched on and have any additional detail added as\n fields.\nSnag(issue: String, cause: List(String))\n\n\nSynonyms:\nsnag.Snag\nsnag Snag","url":"snag.html#Snag"},{"doc":"snag","title":"context","content":"pub fn context(\n  result: Result(a, Snag),\n  issue: String,\n) -&gt; Result(a, Snag)\n Add additional contextual information to a `Snag` wrapped in a `Result`.\n\n # Example\n\n \n &gt; error(&quot;Not enough credit&quot;)\n &gt; |&gt; context(&quot;Unable to make purchase&quot;)\n &gt; |&gt; result.map_error(line_print)\n Error(&quot;error: Unable to make purchase &lt;- Not enough credit&quot;)\n \nSynonyms:\nsnag.context\nsnag context","url":"snag.html#context"},{"doc":"snag","title":"error","content":"pub fn error(issue: String) -&gt; Result(a, Snag)\n Create a new `Snag` wrapped in a `Result` with the given issue text.\n\n # Example\n\n \n &gt; error(&quot;Not enough credit&quot;)\n Error(new(&quot;Not enough credit&quot;))\n \nSynonyms:\nsnag.error\nsnag error","url":"snag.html#error"},{"doc":"snag","title":"layer","content":"pub fn layer(snag: Snag, issue: String) -&gt; Snag\n Add additional contextual information to a `Snag`.\n\n See also the `context` function for adding contextual information to a `Snag`\n wrapped in a `Result`.\n\n # Example\n\n \n &gt; new(&quot;Not enough credit&quot;)\n &gt; |&gt; layer(&quot;Unable to make purchase&quot;)\n &gt; |&gt; line_print\n &quot;error: Unable to make purchase &lt;- Not enough credit&quot;\n \nSynonyms:\nsnag.layer\nsnag layer","url":"snag.html#layer"},{"doc":"snag","title":"line_print","content":"pub fn line_print(snag: Snag) -&gt; String\n Turn a snag into a single-line string, optimised for compactness. This may be\n useful for logging snags.\n\n # Example\n\n \n &gt; new(&quot;Not enough credit&quot;)\n &gt; |&gt; layer(&quot;Unable to make purchase&quot;)\n &gt; |&gt; layer(&quot;Character creation failed&quot;)\n &gt; |&gt; pretty_print\n &quot;error: Character creation failed &lt;- Unable to make purchase &lt;- Not enough credit&quot;\n \nSynonyms:\nsnag.line_print\nsnag line_print","url":"snag.html#line_print"},{"doc":"snag","title":"new","content":"pub fn new(issue: String) -&gt; Snag\n Create a new `Snag` with the given issue text.\n\n See also the `error` function for creating a `Snag` wrapped in a `Result`.\n\n # Example\n\n \n &gt; new(&quot;Not enough credit&quot;)\n &gt; |&gt; line_print\n &quot;error: Not enough credit&quot;\n \nSynonyms:\nsnag.new\nsnag new","url":"snag.html#new"},{"doc":"snag","title":"pretty_print","content":"pub fn pretty_print(snag: Snag) -&gt; String\n Turn a snag into a multi-line string, optimised for readability.\n\n # Example\n\n \n &gt; new(&quot;Not enough credit&quot;)\n &gt; |&gt; layer(&quot;Unable to make purchase&quot;)\n &gt; |&gt; layer(&quot;Character creation failed&quot;)\n &gt; |&gt; pretty_print\n &quot;error: Character creation failed\n\n cause:\n   0: Unable to make purchase\n   1: Not enough credit\n &quot;\n \nSynonyms:\nsnag.pretty_print\nsnag pretty_print","url":"snag.html#pretty_print"},{"doc":"snag","title":"snag","content":"","url":"snag.html"}]);